#ifndef _CONFIG_H
#define _CONFIG_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(__C30__)
#include <libpic30.h>
#endif

#define VERSION   "HPSpin 4.17" 

#include "include/spin2.h"              // configuracao de hardware


//*****************************************************************************
// TCP/IP Config
//
// Define the default values for your MAC and IP configuration.
// Notice that the IP addresses are separated by commas and not by the
// traditional dot in the decimal dotted notation.
//
#define DEFAULT_MAC_ADDRESS { 0x00, 0x04, 0xa3, 0x00, 0x02, 0x00 }
#define DEFAULT_IP_ADDRESS  { 192, 168, 0, 60 }
#define DEFAULT_NETMASK     { 255, 255, 255, 0 }
#define DEFAULT_GATEWAY     { 192, 168, 0, 1 }
#define DEFAULT_NS1         { 192, 168, 1, 1 }
//#define SNTP_SERVER_IP      { 192, 43, 244, 18 }   // time.nist.gov

#define DEFAULT_NETBIOS_NAME  "SPIN2"


//*****************************************************************************
// Stack Modules
//
#define STACK_USE_ICMP               // ICMP reply (ping) module
#define STACK_USE_HTTP_SERVER        // HTTP server
#define STACK_USE_ANNOUNCE           // Ethernet Device Discoverer server/client


//*****************************************************************************
// Some time related definitions
//
// Define Ticks per second for the tick manager
//
#define TICKS_PER_SECOND     (100)   // 10ms


//*****************************************************************************
// Miscellaneous Stack Options
//

// Define the Baud Rate for the RS-232 Serial Interface
#define BAUD_RATE (19200)            // bps

// Include User process code in main.c
//
//#define ENABLE_USER_PROCESS


// Define default TCP port for HTTP server
//
#define HTTP_PORT (80)


// Maximum number of TCP sockets allowed
// Note that each TCP socket consumes 38 bytes of RAM.
//
#define MAX_SOCKETS          (6u)

// Maximum number of TCP sockets allowed
//
#define MAX_UDP_SOCKETS      (4u)

// Maximum numbers of simultaneous HTTP connections allowed.
// Each connection consumes 10 bytes of RAM.
//
#define MAX_HTTP_CONNECTIONS (3u)


//*****************************************************************************
// Storage options
#define MPFS_USE_PGRM            // Use Program Memory


//*****************************************************************************
// Set Baud Rate Generator Register value based on defined baud rate
// and map registers based on UART.
#if ((TCY_CLOCK+2*BAUD_RATE)/BAUD_RATE/4 - 1) <= 255
        #define SPBRG_VAL ((TCY_CLOCK+2*BAUD_RATE)/BAUD_RATE/4 - 1)
        #define USART_USE_BRGH_HIGH
#else
        #define SPBRG_VAL ((TCY_CLOCK+8*BAUD_RATE)/BAUD_RATE/16 - 1)
#endif

#define UART_TXSTAbits TXSTAbits
#define UART_RCSTAbits RCSTAbits
#define UART_TXREG     TXREG
#define UART_TXSTA     TXSTA
#define UART_RCSTA     RCSTA
#define UART_SPBRG     SPBRG
#define UART_RCREG     RCREG

#define STACK_USE_UDP
#define STACK_USE_TCP
#define STACK_USE_SW_CKSUM
#define MAC_TX_BUFFER_SIZE   (576)
#define MAC_TX_BUFFER_COUNT  (MAX_SOCKETS+1)
#define MAC_RX_BUFFER_SIZE       (MAC_TX_BUFFER_SIZE)

//*****************************************************************************
// Based on the protocol/applications modules enabled calculate the number
// of available sockets
#define AVAILABLE_SOCKETS        (MAX_SOCKETS)
#define AVAILABLE_UDP_SOCKETS    (MAX_UDP_SOCKETS)

#define AVAILABLE_SOCKETS2   (AVAILABLE_SOCKETS - MAX_HTTP_CONNECTIONS)

#define MPFS_RESERVE_BLOCK       (64)


//*****************************************************************************
// Verify hardware configuration

// Verify that CPU clock frequency is defined
#if !defined(CPU_CLOCK)
    #error CFG002: CPU_CLOCK not defined in hardware configuration file
#endif

#if !defined(TCY_CLOCK)
    #error CFG003: TCY_CLOCK not defined in hardware configuration file
#endif

#if (SPBRG_VAL > 255) && !defined(__C30__)
    #error CFG004: SPBRG value is out of range for defined CPU_CLOCK
#endif

#if !defined(TICKS_PER_SECOND)
    #error CFG005: TICKS_PER_SECOND must be defined
#endif

// Verify Ethernet controller interface configuration
#if defined(USE_ENC28J60)
    #if !defined(ENC_CS_IO)
        #error CFG011: ENC_CS_IO not defined in hardware configuration file
    #endif
    #if !defined(ENC_SPI_IF) || !defined(ENC_SSPBUF) || !defined(ENC_SPISTAT) || \
        !defined(ENC_SPISTATbits) || !defined(ENC_SPICON1) || \
        !defined(ENC_SPICON1bits) 
        #error CFG012: Missing SPI configuration registers for ENC28J60
    #endif
#else
    #error CFG010: No ethernet controller defined
#endif



//*****************************************************************************
// Verify TCP/IP stack options and configuration
//
#if !defined(DEFAULT_MAC_ADDRESS)
    #error CFG050: No MAC Address defined
#endif

#if !defined(DEFAULT_IP_ADDRESS) 
    #error CFG051: No IP Address defined
#endif

#if !defined(DEFAULT_NETMASK)
    #error CFG052: No network mask defined
#endif

#if !defined(DEFAULT_GATEWAY)
    #error CFG053: No default gateway defined
#endif

#if defined(STACK_USE_DNS) && !defined(DEFAULT_NS1)
    #error CFG054: DNS client enabled but no default name server defined
#endif

#if defined(STACK_USE_FTP_SERVER) && (!defined(FTP_USERNAME) || \
    !defined(FTP_PASSWORD))
    #error CFG055: FTP server enabled but username or password not defined
#endif

#if defined(STACK_USE_NBNS) && !defined(DEFAULT_NETBIOS_NAME)
    #error CFG056: NetBIOS Name Service enabled but hostname not defined
#endif

#if (MAX_SOCKETS <= 0 || MAX_SOCKETS > 255)
    #error CFG058: Invalid MAX_SOCKETS value defined
#endif

#if (MAX_UDP_SOCKETS <= 0 || MAX_UDP_SOCKETS > 255 )
    #error CFG059: Invalid MAX_UDP_SOCKETS value defined
#endif

#if (MAC_TX_BUFFER_SIZE <= 0 || MAC_TX_BUFFER_SIZE > 1500 )
    #error CFG060: Invalid MAC_TX_BUFFER_SIZE value defined
#endif

#if ( (MAC_TX_BUFFER_SIZE * MAC_TX_BUFFER_COUNT) > (5 * 1024) )
    #error CFG061: Warning, receive buffer is small.  Excessive packet loss may occur.
#endif

#if defined(STACK_USE_HTTP_SERVER)
    #if (MAX_HTTP_CONNECTIONS <= 0 || MAX_HTTP_CONNECTIONS > 255 )
        #error CFG062: Invalid MAX_HTTP_CONNECTIONS value defined
    #endif
#endif

#if AVAILABLE_SOCKETS3 < 0 || AVAILABLE_SOCKETS3 > MAX_SOCKETS
    #error CFG063: The defined MAX_SOCKETS value is too low
#endif

#if AVAILABLE_UDP_SOCKETS4 < 0 || AVAILABLE_UDP_SOCKETS4 > MAX_UDP_SOCKETS
    #error CFG064: The defined MAX_UDP_SOCKETS value is too low
#endif

#if defined(MPFS_USE_EEPROM) && defined(MPFS_USE_PGRM)
    #error CFG080: You can not define both clases of storage defined
#endif

#if defined(STACK_USE_HTTP_SERVER) && !(defined(MPFS_USE_EEPROM) || \
    defined(MPFS_USE_PGRM))
    #error CFG081: No storage option defined for HTTP Server pages
#endif

#if defined(STACK_USE_FTP_SERVER) && defined(MPFS_USE_PGRM)
    #error CFG082: FTP Server not supported for program memory storage option
#endif

#if defined(STACK_USE_HTTP_SERVER)
    #if (MAC_TX_BUFFER_SIZE <= 130 || MAC_TX_BUFFER_SIZE > 1500 )
        #error HTTP001: Invalid MAC_TX_BUFFER_SIZE value specified
    #endif
#endif


//*****************************************************************************
// Compiler specific replacement macros
//*****************************************************************************
#if defined(DEBUG)
    #define DebugPrint(a)       {putrsUART(a);}
#else
    #define DebugPrint(a)       {}
#endif

#if defined(__18CXX)
    #define ROM                  rom
    #define ROMPC                rom far char
    #define strcpypgm2ram(a, b)  strcpypgm2ram(a,(rom far char*)b)
    #define memcpypgm2ram(a,b,c) memcpypgm2ram((void *)a,(const far rom void *)b,c)
    #define memcmppgm2ram(a,b,c) memcmppgm2ram((void *)a,(const far rom void *)b,c)
    #define __attribute__(a)
    #define BusyUART()           BusyUSART()
    #define CloseUART()          CloseUSART()
    #define ConfigIntUART(a)     ConfigIntUSART(a)
    #define DataRdyUART()        DataRdyUSART()
    #define OpenUART(a,b,c)      OpenUSART(a,b,c)
    #define ReadUART()           ReadUSART()
    #define WriteUART(a)         WriteUSART(a)
    #define getsUART(a,b,c)      getsUSART(b,a)
    #define putsUART(a)          putsUSART(a)
    #define getcUART()           ReadUSART()
    #define putcUART(a)          WriteUSART(a)
    #define putrsUART(a)         putrsUSART((rom far char*)a)
    #define printf(a)            printf((rom far char *)a) 
#endif

#endif // _CONFIG_H

